@@ -13,10 +13,11 @@
 
 #include "core/defs.h"
 #include "core/list.h"
+#include "core/reap.h"
 #include "core/taskq.h"
 #include "core/thread.h"
 
-typedef void (*nni_aio_cancelfn)(nni_aio *, void *, int);
+typedef void (*nni_aio_cancel_fn)(nni_aio *, void *, int);
 
 // nni_aio_init initializes an aio object.  The callback is called with
 // the supplied argument when the operation is complete.  If NULL is
@@ -28,6 +29,10 @@ extern void nni_aio_init(nni_aio *, nni_cb, void *arg);
 // It waits for the callback to complete.
 extern void nni_aio_fini(nni_aio *);
 
+// nni_aio_reap is used to asynchronously reap the aio.  It can
+// be called even from the callback of the aio itself.
+extern void nni_aio_reap(nni_aio *);
+
 // nni_aio_alloc allocates an aio object and initializes it.  The callback
 // is called with the supplied argument when the operation is complete.
 // If NULL is supplied for the callback, then nni_aio_wake is used in its
@@ -35,7 +40,7 @@ extern void nni_aio_fini(nni_aio *);
 extern int nni_aio_alloc(nni_aio **, nni_cb, void *arg);
 
 // nni_aio_free frees the aio, releasing resources (locks)
-// associated with it. This is safe to call on zero'd memory.
+// associated with it. This is safe to call on zeroed memory.
 // This must only be called on an object that was allocated
 // with nni_aio_allocate.
 extern void nni_aio_free(nni_aio *aio);
@@ -43,8 +48,8 @@ extern void nni_aio_free(nni_aio *aio);
 // nni_aio_stop cancels any unfinished I/O, running completion callbacks,
 // but also prevents any new operations from starting (nni_aio_start will
 // return NNG_ESTATE).  This should be called before nni_aio_free().  The
-// best pattern is to call nni_aio_stop on all linked aios, before calling
-// nni_aio_free on any of them.  This function will block until any
+// best pattern is to call nni_aio_stop on all linked aio objects, before
+// calling nni_aio_free on any of them.  This function will block until any
 // callbacks are executed, and therefore it should never be executed
 // from a callback itself.  (To abort operations without blocking
 // use nni_aio_cancel instead.)
@@ -53,20 +58,10 @@ extern void nni_aio_stop(nni_aio *);
 // nni_aio_close closes the aio for further activity. It aborts any in-progress
 // transaction (if it can), and future calls nni_aio_begin or nni_aio_schedule
 // with both result in NNG_ECLOSED. The expectation is that protocols call this
-// for all their aios in a stop routine, before calling fini on any of them.
+// for all their aio objects in a stop routine, before calling fini on any of
+// them.
 extern void nni_aio_close(nni_aio *);
 
-// nni_aio_set_data sets user data.  This should only be done by the
-// consumer, initiating the I/O.  The intention is to be able to store
-// additional data for use when the operation callback is executed.
-// The index represents the "index" at which to store the data.  A maximum
-// of 4 elements can be stored with the (index < 4).
-extern void nni_aio_set_data(nni_aio *, unsigned, void *);
-
-// nni_aio_get_data returns the user data that was previously stored
-// with nni_aio_set_data.
-extern void *nni_aio_get_data(nni_aio *, unsigned);
-
 // nni_set_input sets input parameters on the AIO.  The semantic details
 // of this will be determined by the specific AIO operation.  AIOs can
 // carry up to 4 input parameters.
@@ -117,11 +112,11 @@ extern int  nni_aio_list_active(nni_aio *);
 
 // nni_aio_finish is called by the provider when an operation is complete.
 extern void nni_aio_finish(nni_aio *, int, size_t);
-// nni_aio_finish_synch is to be called when a synchronous completion is
+// nni_aio_finish_sync is to be called when a synchronous completion is
 // desired.  It is very important that the caller not hold any locks when
 // calling this, but it is useful for chaining completions to minimize
 // context switch overhead during completions.
-extern void nni_aio_finish_synch(nni_aio *, int, size_t);
+extern void nni_aio_finish_sync(nni_aio *, int, size_t);
 extern void nni_aio_finish_error(nni_aio *, int);
 extern void nni_aio_finish_msg(nni_aio *, nni_msg *);
 
@@ -145,7 +140,7 @@ extern void  nni_aio_set_prov_extra(nni_aio *, unsigned, void *);
 // i.e. if the count refers to more data than the iov can support, then
 // the result will be left over count.
 extern size_t nni_aio_iov_advance(nni_aio *, size_t);
-// nni_aio_iov_count returns the number of bytes referenced by the aio's iov.
+// nni_aio_iov_count returns the number of bytes referenced by the aio iov.
 extern size_t nni_aio_iov_count(nni_aio *);
 
 extern int nni_aio_set_iov(nni_aio *, unsigned, const nni_iov *);
@@ -155,24 +150,23 @@ extern void nni_aio_get_iov(nni_aio *, unsigned *, nni_iov **);
 extern void nni_aio_normalize_timeout(nni_aio *, nng_duration);
 extern void nni_aio_bump_count(nni_aio *, size_t);
 
-extern void nni_aio_set_sockaddr(nni_aio *aio, const nng_sockaddr *);
-extern void nni_aio_get_sockaddr(nni_aio *aio, nng_sockaddr *);
-
 // nni_aio_schedule indicates that the AIO has begun, and is scheduled for
-// asychronous completion. This also starts the expiration timer. Note that
-// prior to this, the aio is uncancellable.  If the operation has a zero
+// asynchronous completion. This also starts the expiration timer. Note that
+// prior to this, the aio cannot be canceled.  If the operation has a zero
 // timeout (NNG_FLAG_NONBLOCK) then NNG_ETIMEDOUT is returned.  If the
 // operation has already been canceled, or should not be run, then an error
 // is returned.  (In that case the caller should probably either return an
 // error to its caller, or possibly cause an asynchronous error by calling
 // nni_aio_finish_error on this aio.)
-extern int nni_aio_schedule(nni_aio *, nni_aio_cancelfn, void *);
+extern int nni_aio_schedule(nni_aio *, nni_aio_cancel_fn, void *);
 
 extern void nni_sleep_aio(nni_duration, nni_aio *);
 
 extern int  nni_aio_sys_init(void);
 extern void nni_aio_sys_fini(void);
 
+typedef struct nni_aio_expire_q nni_aio_expire_q;
+
 // An nni_aio is an async I/O handle.  The details of this aio structure
 // are private to the AIO framework.  The structure has the public name
 // (nng_aio) so that we minimize the pollution in the public API namespace.
@@ -191,15 +185,11 @@ struct nng_aio {
 
 	// Read/write operations.
 	nni_iov  a_iov[8];
-	unsigned a_niov;
+	unsigned a_nio;
 
 	// Message operations.
 	nni_msg *a_msg;
 
-	// User scratch data.  Consumers may store values here, which
-	// must be preserved by providers and the framework.
-	void *a_user_data[4];
-
 	// Operation inputs & outputs.  Up to 4 inputs and 4 outputs may be
 	// specified.  The semantics of these will vary, and depend on the
 	// specific operation.
@@ -207,19 +197,14 @@ struct nng_aio {
 	void *a_outputs[4];
 
 	// Provider-use fields.
-	nni_aio_cancelfn a_cancel_fn;
-	void *           a_cancel_arg;
-	nni_list_node    a_prov_node;     // Linkage on provider list.
-	void *           a_prov_extra[4]; // Extra data used by provider
+	nni_aio_cancel_fn a_cancel_fn;
+	void *            a_cancel_arg;
+	nni_list_node     a_prov_node;     // Linkage on provider list.
+	void *            a_prov_extra[2]; // Extra data used by provider
 
-	// Socket address.  This turns out to be very useful, as we wind up
-	// needing socket addresses for numerous connection related routines.
-	// It would be cleaner to not have this and avoid burning the space,
-	// but having this hear dramatically simplifies lots of code.
-	nng_sockaddr a_sockaddr;
-
-	// Expire node.
-	nni_list_node a_expire_node;
+	nni_aio_expire_q *a_expire_q;
+	nni_list_node     a_expire_node; // Expiration node
+	nni_reap_node     a_reap_node;
 };
 
 #endif // CORE_AIO_H
