@@ -169,7 +169,7 @@ tcp_dialer_cb(nni_posix_pfd *pfd, unsigned ev, void *arg)
 // We don't give local address binding support.  Outbound dialers always
 // get an ephemeral port.
 void
-nni_tcp_dial(nni_tcp_dialer *d, nni_aio *aio)
+nni_tcp_dial(nni_tcp_dialer *d, const nni_sockaddr *sa, nni_aio *aio)
 {
 	nni_tcp_conn *          c;
 	nni_posix_pfd *         pfd = NULL;
@@ -179,14 +179,12 @@ nni_tcp_dial(nni_tcp_dialer *d, nni_aio *aio)
 	int                     rv;
 	int                     ka;
 	int                     nd;
-	nng_sockaddr            sa;
 
 	if (nni_aio_begin(aio) != 0) {
 		return;
 	}
 
-	nni_aio_get_sockaddr(aio, &sa);
-	if (((sslen = nni_posix_nn2sockaddr(&ss, &sa)) == 0) ||
+	if (((sslen = nni_posix_nn2sockaddr(&ss, sa)) == 0) ||
 	    ((ss.ss_family != AF_INET) && (ss.ss_family != AF_INET6))) {
 		nni_aio_finish_error(aio, NNG_EADDRINVAL);
 		return;
@@ -323,7 +321,7 @@ tcp_dialer_get_locaddr(void *arg, void *buf, size_t *szp, nni_type t)
 	nng_sockaddr    sa;
 
 	nni_mtx_lock(&d->mtx);
-	if (nni_posix_sockaddr2nn(&sa, &d->src) != 0) {
+	if (nni_posix_sockaddr2nn(&sa, &d->src, d->srclen) != 0) {
 		sa.s_family = NNG_AF_UNSPEC;
 	}
 	nni_mtx_unlock(&d->mtx);
@@ -338,13 +336,13 @@ tcp_dialer_set_locaddr(void *arg, const void *buf, size_t sz, nni_type t)
 	struct sockaddr_storage ss;
 	struct sockaddr_in *    sin;
 	struct sockaddr_in6 *   sin6;
-	size_t                  sslen;
+	size_t                  len;
 	int                     rv;
 
 	if ((rv = nni_copyin_sockaddr(&sa, buf, sz, t)) != 0) {
 		return (rv);
 	}
-	if ((sslen = nni_posix_nn2sockaddr(&ss, &sa)) == 0) {
+	if ((len = nni_posix_nn2sockaddr(&ss, &sa)) == 0) {
 		return (NNG_EADDRINVAL);
 	}
 	// Ensure we are either IPv4 or IPv6, and port is not set.  (We
@@ -372,7 +370,7 @@ tcp_dialer_set_locaddr(void *arg, const void *buf, size_t sz, nni_type t)
 			return (NNG_ECLOSED);
 		}
 		d->src    = ss;
-		d->srclen = sslen;
+		d->srclen = len;
 		nni_mtx_unlock(&d->mtx);
 	}
 	return (0);
@@ -400,14 +398,14 @@ static const nni_option tcp_dialer_options[] = {
 };
 
 int
-nni_tcp_dialer_getopt(
+nni_tcp_dialer_get(
     nni_tcp_dialer *d, const char *name, void *buf, size_t *szp, nni_type t)
 {
 	return (nni_getopt(tcp_dialer_options, name, d, buf, szp, t));
 }
 
 int
-nni_tcp_dialer_setopt(nni_tcp_dialer *d, const char *name, const void *buf,
+nni_tcp_dialer_set(nni_tcp_dialer *d, const char *name, const void *buf,
     size_t sz, nni_type t)
 {
 	return (nni_setopt(tcp_dialer_options, name, d, buf, sz, t));
