@@ -78,7 +78,7 @@ typedef struct {
 	size_t                  tcp_send_len;
 	size_t                  tcp_send_head;
 	size_t                  tcp_send_tail;
-	struct nni_reap_item    reap;
+	nni_reap_node           reap;
 
 	// ... engine connection data follows
 } tls_conn;
@@ -89,10 +89,16 @@ static void tls_do_send(tls_conn *);
 static void tls_do_recv(tls_conn *);
 static void tls_tcp_send_start(tls_conn *);
 static void tls_free(void *);
+static void tls_reap(void *);
 static int  tls_alloc(tls_conn **, nng_tls_config *, nng_aio *);
 static int  tls_start(tls_conn *, nng_stream *);
 static void tls_tcp_error(tls_conn *, int);
 
+static nni_reap_list tls_conn_reap_list = {
+	.rl_offset = offsetof(tls_conn, reap),
+	.rl_func   = tls_reap,
+};
+
 typedef struct {
 	nng_stream_dialer  ops;
 	nng_stream_dialer *d; // underlying TCP dialer
@@ -153,7 +159,9 @@ static void
 tls_conn_cancel(nni_aio *aio, void *arg, int rv)
 {
 	tls_conn *conn = arg;
-	NNI_ASSERT(conn->user_aio == aio);
+
+	NNI_ARG_UNUSED(aio);
+
 	// Just pass this down.  If the connection is already done, this
 	// will have no effect.
 	nni_aio_abort(&conn->conn_aio, rv);
@@ -186,13 +194,18 @@ tls_dialer_dial(void *arg, nng_aio *aio)
 static int
 tls_check_string(const void *v, size_t sz, nni_opt_type t)
 {
-	if ((t != NNI_TYPE_OPAQUE) && (t != NNI_TYPE_STRING)) {
+	switch (t) {
+	case NNI_TYPE_OPAQUE:
+		if (nni_strnlen(v, sz) >= sz) {
+			return (NNG_EINVAL);
+		}
+		return (0);
+	case NNI_TYPE_STRING:
+		// Caller is assumed to pass a good string.
+		return (0);
+	default:
 		return (NNG_EBADTYPE);
 	}
-	if (nni_strnlen(v, sz) >= sz) {
-		return (NNG_EINVAL);
-	}
-	return (0);
 }
 
 static int
@@ -324,13 +337,12 @@ static const nni_option tls_dialer_opts[] = {
 };
 
 static int
-tls_dialer_getx(
-    void *arg, const char *name, void *buf, size_t *szp, nni_type t)
+tls_dialer_get(void *arg, const char *name, void *buf, size_t *szp, nni_type t)
 {
 	tls_dialer *d = arg;
 	int         rv;
 
-	rv = nni_stream_dialer_getx(d->d, name, buf, szp, t);
+	rv = nni_stream_dialer_get(d->d, name, buf, szp, t);
 	if (rv == NNG_ENOTSUP) {
 		rv = nni_getopt(tls_dialer_opts, name, d, buf, szp, t);
 	}
@@ -338,13 +350,13 @@ tls_dialer_getx(
 }
 
 static int
-tls_dialer_setx(
+tls_dialer_set(
     void *arg, const char *name, const void *buf, size_t sz, nni_type t)
 {
 	tls_dialer *d = arg;
 	int         rv;
 
-	rv = nni_stream_dialer_setx(d->d, name, buf, sz, t);
+	rv = nni_stream_dialer_set(d->d, name, buf, sz, t);
 	if (rv == NNG_ENOTSUP) {
 		rv = nni_setopt(tls_dialer_opts, name, d, buf, sz, t);
 	}
@@ -389,8 +401,8 @@ nni_tls_dialer_alloc(nng_stream_dialer **dp, const nng_url *url)
 	d->ops.sd_close = tls_dialer_close;
 	d->ops.sd_free  = tls_dialer_free;
 	d->ops.sd_dial  = tls_dialer_dial;
-	d->ops.sd_getx  = tls_dialer_getx;
-	d->ops.sd_setx  = tls_dialer_setx;
+	d->ops.sd_get   = tls_dialer_get;
+	d->ops.sd_set   = tls_dialer_set;
 	*dp             = (void *) d;
 	return (rv);
 }
@@ -584,13 +596,13 @@ static const nni_option tls_listener_opts[] = {
 };
 
 static int
-tls_listener_getx(
+tls_listener_get(
     void *arg, const char *name, void *buf, size_t *szp, nni_type t)
 {
 	int           rv;
 	tls_listener *l = arg;
 
-	rv = nni_stream_listener_getx(l->l, name, buf, szp, t);
+	rv = nni_stream_listener_get(l->l, name, buf, szp, t);
 	if (rv == NNG_ENOTSUP) {
 		rv = nni_getopt(tls_listener_opts, name, l, buf, szp, t);
 	}
@@ -598,13 +610,13 @@ tls_listener_getx(
 }
 
 static int
-tls_listener_setx(
+tls_listener_set(
     void *arg, const char *name, const void *buf, size_t sz, nni_type t)
 {
 	int           rv;
 	tls_listener *l = arg;
 
-	rv = nni_stream_listener_setx(l->l, name, buf, sz, t);
+	rv = nni_stream_listener_set(l->l, name, buf, sz, t);
 	if (rv == NNG_ENOTSUP) {
 		rv = nni_setopt(tls_listener_opts, name, l, buf, sz, t);
 	}
@@ -647,80 +659,12 @@ nni_tls_listener_alloc(nng_stream_listener **lp, const nng_url *url)
 	l->ops.sl_close  = tls_listener_close;
 	l->ops.sl_accept = tls_listener_accept;
 	l->ops.sl_listen = tls_listener_listen;
-	l->ops.sl_getx   = tls_listener_getx;
-	l->ops.sl_setx   = tls_listener_setx;
+	l->ops.sl_get    = tls_listener_get;
+	l->ops.sl_set    = tls_listener_set;
 	*lp              = (void *) l;
 	return (0);
 }
 
-// The following checks exist for socket configuration, when we need to
-// configure an option on a socket before any transport is configured
-// underneath.
-
-static int
-tls_check_config(const void *buf, size_t sz, nni_type t)
-{
-	int             rv;
-	nng_tls_config *cfg;
-
-	if ((rv = nni_copyin_ptr((void **) &cfg, buf, sz, t)) != 0) {
-		return (rv);
-	}
-	if (cfg == NULL) {
-		return (NNG_EINVAL);
-	}
-	return (0);
-}
-
-static int
-tls_check_auth_mode(const void *buf, size_t sz, nni_type t)
-{
-	int mode;
-	int rv;
-
-	rv = nni_copyin_int(&mode, buf, sz, NNG_TLS_AUTH_MODE_NONE,
-	    NNG_TLS_AUTH_MODE_REQUIRED, t);
-	return (rv);
-}
-
-static const nni_chkoption tls_check_opts[] = {
-	{
-	    .o_name  = NNG_OPT_TLS_CONFIG,
-	    .o_check = tls_check_config,
-	},
-	{
-	    .o_name  = NNG_OPT_TLS_SERVER_NAME,
-	    .o_check = tls_check_string,
-	},
-	{
-	    .o_name  = NNG_OPT_TLS_CA_FILE,
-	    .o_check = tls_check_string,
-	},
-	{
-	    .o_name  = NNG_OPT_TLS_CERT_KEY_FILE,
-	    .o_check = tls_check_string,
-	},
-	{
-	    .o_name  = NNG_OPT_TLS_AUTH_MODE,
-	    .o_check = tls_check_auth_mode,
-	},
-	{
-	    .o_name = NULL,
-	},
-};
-
-int
-nni_tls_checkopt(const char *name, const void *data, size_t sz, nni_type t)
-{
-	int rv;
-
-	rv = nni_chkopt(tls_check_opts, name, data, sz, t);
-	if (rv == NNG_ENOTSUP) {
-		rv = nni_stream_checkopt("tcp", name, data, sz, t);
-	}
-	return (rv);
-}
-
 static void
 tls_cancel(nni_aio *aio, void *arg, int rv)
 {
@@ -824,7 +768,7 @@ static const nni_option tls_options[] = {
 };
 
 static int
-tls_setx(void *arg, const char *name, const void *buf, size_t sz, nni_type t)
+tls_set(void *arg, const char *name, const void *buf, size_t sz, nni_type t)
 {
 	tls_conn *  conn = arg;
 	int         rv;
@@ -832,19 +776,19 @@ tls_setx(void *arg, const char *name, const void *buf, size_t sz, nni_type t)
 
 	tcp = (conn != NULL) ? conn->tcp : NULL;
 
-	if ((rv = nni_stream_setx(tcp, name, buf, sz, t)) != NNG_ENOTSUP) {
+	if ((rv = nni_stream_set(tcp, name, buf, sz, t)) != NNG_ENOTSUP) {
 		return (rv);
 	}
 	return (nni_setopt(tls_options, name, conn, buf, sz, t));
 }
 
 static int
-tls_getx(void *arg, const char *name, void *buf, size_t *szp, nni_type t)
+tls_get(void *arg, const char *name, void *buf, size_t *szp, nni_type t)
 {
 	tls_conn *conn = arg;
 	int       rv;
 
-	if ((rv = nni_stream_getx(conn->tcp, name, buf, szp, t)) !=
+	if ((rv = nni_stream_get(conn->tcp, name, buf, szp, t)) !=
 	    NNG_ENOTSUP) {
 		return (rv);
 	}
@@ -891,8 +835,8 @@ tls_alloc(tls_conn **conn_p, nng_tls_config *cfg, nng_aio *user_aio)
 	conn->stream.s_free  = tls_free;
 	conn->stream.s_send  = tls_send;
 	conn->stream.s_recv  = tls_recv;
-	conn->stream.s_getx  = tls_getx;
-	conn->stream.s_setx  = tls_setx;
+	conn->stream.s_get   = tls_get;
+	conn->stream.s_set   = tls_set;
 
 	nng_tls_config_hold(cfg);
 	*conn_p = conn;
@@ -934,7 +878,7 @@ tls_free(void *arg)
 {
 	tls_conn *conn = arg;
 
-	nni_reap(&conn->reap, tls_reap, conn);
+	nni_reap(&tls_conn_reap_list, conn);
 }
 
 static int
@@ -1098,6 +1042,8 @@ tls_tcp_send_cb(void *arg)
 	count = nni_aio_count(aio);
 	NNI_ASSERT(count <= conn->tcp_send_len);
 	conn->tcp_send_len -= count;
+	conn->tcp_send_tail += count;
+	conn->tcp_send_tail %= NNG_TLS_MAX_SEND_SIZE;
 	tls_tcp_send_start(conn);
 
 	if (tls_do_handshake(conn)) {
@@ -1180,6 +1126,7 @@ tls_tcp_send_start(tls_conn *conn)
 
 	while (len > 0) {
 		size_t cnt;
+		NNI_ASSERT(nio < 2);
 		if (tail < head) {
 			cnt = head - tail;
 		} else {
@@ -1196,7 +1143,6 @@ tls_tcp_send_start(tls_conn *conn)
 		nio++;
 	}
 	conn->tcp_send_active = true;
-	conn->tcp_send_tail   = tail;
 	nni_aio_set_iov(&conn->tcp_send, nio, iov);
 	nng_stream_send(conn->tcp, &conn->tcp_send);
 }
@@ -1228,6 +1174,8 @@ nng_tls_engine_send(void *arg, const uint8_t *buf, size_t *szp)
 	// We are committed at this point to sending out len bytes.
 	// Update this now, so that we can use len to update.
 	*szp = len;
+	conn->tcp_send_len += len;
+	NNI_ASSERT(conn->tcp_send_len <= NNG_TLS_MAX_SEND_SIZE);
 
 	while (len > 0) {
 		if (head >= tail) {
@@ -1243,11 +1191,11 @@ nng_tls_engine_send(void *arg, const uint8_t *buf, size_t *szp)
 		buf += cnt;
 		head += cnt;
 		head %= NNG_TLS_MAX_SEND_SIZE;
-		conn->tcp_send_len += cnt;
-		conn->tcp_send_head = head;
 		len -= cnt;
 	}
 
+	conn->tcp_send_head = head;
+
 	tls_tcp_send_start(conn);
 	return (0);
 }
@@ -1525,7 +1473,7 @@ nng_tls_engine_register(const nng_tls_engine *engine)
 extern int NNG_TLS_ENGINE_INIT(void);
 #else
 static int
-NNI_TLS_ENGINE_INIT(void)
+NNG_TLS_ENGINE_INIT(void)
 {
 	return (0);
 }
@@ -1558,6 +1506,7 @@ nni_tls_sys_init(void)
 void
 nni_tls_sys_fini(void)
 {
+	nni_reap_drain();
 	NNG_TLS_ENGINE_FINI();
 }
 
